﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DeaneBarker.Kiwi
{
    public class Lines : List<string>
    {
        private int current = -1;

        public Lines(string input)
        {
            if(string.IsNullOrWhiteSpace(input))
            {
                return;
            }

            input.Split(new[] { "\r\n", "\r", "\n" }, StringSplitOptions.None).ToList().ForEach(l =>
            {
                if (string.IsNullOrWhiteSpace(l))
                {
                    Add(string.Empty); // We trim empty lines, nothing else
                }
                else
                {
                    Add(l);
                }
            });
        }

        public string Current => current > this.Count() - 1 || current == -1 ? null : this[current];
        public int CurrentLineNumber => current + 1;
        public List<string> Remaining => current == -1 || !this.Any() ? this : GetRange(current, this.Count() - current).ToList();

        public string GetNext()
        {
            current++;
            return Current;
        }

        public void Reset()
        {
            current = -1;
        }

        public string PeekNext()
        {
            return PeekOffset(1);
        }

        public string PeekOffset(int offset)
        {
            if(OffsetOutOfBounds(offset))
                return null;

            return this[current + offset];
        }

        public bool? TestNext(Func<string, bool> func)
        {
            if (OffsetOutOfBounds(1))
                return null;

            return (bool?)TestOffset(1,func);
        }

        public bool? TestOffset(int offset, Func<string, bool> func)
        {
            if (OffsetOutOfBounds(offset))
                return null;

            return (bool?)func(PeekOffset(offset));
        }

        public bool? IsEqualsNext(string value)
        {
            if (OffsetOutOfBounds(1))
                return null;

            return IsEqualsOffset(1, value);
        }

        public bool? IsEqualsOffset(int offset, string value)
        {
            if (OffsetOutOfBounds(offset))
                return null;

            return PeekOffset(offset) == value;
        }

        public bool? ContainsNext(string value)
        {
            if (OffsetOutOfBounds(1))
                return null;

            return ContainsOffset(1, value);
        }

        public bool? ContainsOffset(int offset, string value)
        {
            if (OffsetOutOfBounds(offset))
                return null;

            return PeekOffset(offset).Contains(value);
        }

        public bool EqualsAny(string value)
        {
            return this.Any(s => s == value);
        }

        public bool ContainsAny(string value)
        {
            return this.Any(s => s.Contains(value));
        }

        public bool EqualsRemaining(string value)
        {
            return Remaining.Any(s => s == value);
        }

        public bool ContainsRemaining(string value)
        {
            return Remaining.Any(s => s.Contains(value));
        }

        public void RemoveBeforeCurrent()
        {
            this.RemoveRange(0, current);
            current = -1;
        }


        private bool OffsetOutOfBounds(int offset)
        {
            if (current + offset > this.Count() - 1 || current + offset < 0)
            {
                return true;
            }
            return false;
        }


        public new string ToString()
        {
            return String.Join(Environment.NewLine, this);
        }

        public string ToStringRemaining()
        {
            return String.Join(Environment.NewLine, Remaining);
        }

    }
}
