﻿using System.Linq;

namespace DeaneBarker.Kiwi.Parsing
{
    internal static class SectionParser
    {
        internal static Section ParseSection(SectionSource source)
        {
            var name = string.Empty;

            // Does this section have an explicit name on the first line?
            if (source.Lines.FirstOrDefault()?.StartsWith(Settings.Instance.SectionStart) ?? false)
            {
                name = source.Lines.First()
                    .Replace(Settings.Instance.SectionStart, string.Empty)
                    .Replace(Settings.Instance.SectionEnd, string.Empty)
                    .ToLower()
                    .Trim();

                // Got the name; get rid of the line
                source.RemoveFirstLine();
            }

            // Remove any lines that are section directives (this should remove any closing directives in the last line
            source.RemoveSectionDirectives();

            // If it's the default section pointer, reset it to the default name
            name = name == Settings.Instance.DefaultSectionNamePointer ? Settings.Instance.DefaultSectionName : name;

            var section = new Section(
                name, // Might be null
                content: source.Source,
                null, // Gets set in the owning method
                sourceLineNumber: source.SourceLineNumber
            )
            {
                LeftOpen = !source.Lines.Any(l => l.Trim().StartsWith(Settings.Instance.SectionStart) && l.Contains("/"))
            };

            return section;
        }
        
    }
}
