﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DeaneBarker.Kiwi.Parsing
{
    internal class SectionSource
    {
        public List<string> Lines { get; set; }
        public int SourceLineNumber { get; set; }
        public string Source => string.Join(Environment.NewLine, Lines);

        public SectionSource(IEnumerable<string> lines, int sourceLineNumber)
        {
            SourceLineNumber = sourceLineNumber;

            // Trim empty lines from the start
            Lines = lines.SkipWhile(line => string.IsNullOrWhiteSpace(line)).ToList();

            // Trim empty lines from the end
            lines = lines.Reverse().SkipWhile(l => string.IsNullOrWhiteSpace(l)).Reverse().ToList();
        }

        public void RemoveFirstLine()
        {
            Lines = Lines.Skip(1).ToList();
        }

        public void RemoveSectionDirectives()
        {
            RemoveLines(l => 
                l.Trim().ToLower().StartsWith(Settings.Instance.SectionStart) &&
                l.Trim().ToLower().EndsWith(Settings.Instance.SectionEnd)
            );
        }

        // Criteria returns "true" if line should be removed
        public void RemoveLines(Func<string, bool> criteria)
        {
            Lines = Lines.Where(l => !criteria(l)).ToList();
        }
    }
}

