﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DeaneBarker.Kiwi.Parsing
{
    internal static class SourceParser
    {
        public static List<SectionSource> GetSectionSources(string input)
        {
            // This is a hack to support the old delimeter style
            // Someday I'll be able to remove this...
            input = input.Replace("[[[", "[[").Replace("]]]", "]]");

            // Get the lines
            var lines = input.Split(new[] { "\r\n", "\r", "\n" }, StringSplitOptions.None).ToList();

            // If there's no leading section, add the default
            if (lines.FirstOrDefault() != null && !lines.First().Trim().StartsWith(Settings.Instance.SectionStart))
            {
                lines.Insert(0, $"{Settings.Instance.SectionStart} {Settings.Instance.DefaultSectionName} {Settings.Instance.SectionEnd}");
            }

            // Here, we will store the starting line for every new section
            var sectionHeadLines = new Queue<int>();

            var numberedLines = lines.Select((x, i) => new { Value = x.Trim(), Number = i });
            foreach (var line in numberedLines)
            {
                if (line.Value.StartsWith("[[") && line.Value.EndsWith("]]"))
                {
                    if (line.Value.Contains("/"))
                    {
                        sectionHeadLines.Enqueue(line.Number + 1);
                    }
                    else
                    {
                        sectionHeadLines.Enqueue(line.Number);
                    }
                }
            }

            // Here, we will extract the corresponding sets of lines for each section
            var sectionChunks = new List<SectionSource>();
            while (true)
            {
                if (sectionHeadLines.Count == 0)
                {
                    break;
                }

                var sectionStartline = sectionHeadLines.Dequeue();
                var sectionEndline = sectionHeadLines.Count == 0 ? lines.Count() : sectionHeadLines.Peek();
                var sectionLines = lines.Skip(sectionStartline).Take(sectionEndline - sectionStartline).ToList();

                if (!sectionLines.Any(l => !string.IsNullOrWhiteSpace(l)))
                {
                    // This chunk has no content
                    continue;
                }

                sectionChunks.Add(new SectionSource(sectionLines, sectionStartline + 1));
            }

            return sectionChunks;
        }
    }
}

